<?php

/**
 * Copyright (C) 2005-2006 Tino Reichardt
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License Version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * - small guestbook for mediawikis
 */

if (!defined('MEDIAWIKI')) die();

$wgExtensionFunctions[] = "wfGuestbookSpecialpage";
$wgExtensionCredits['specialpage'][] = array(
  'name' => 'Guestbook',
  'author' => 'Tino Reichardt',
  'url' => 'http://www.mcmilk.de/wiki',
);

function wfGuestbookSpecialpage() {
  global $IP, $wgMessageCache;
  require_once( "$IP/includes/SpecialPage.php" );

  /* here we define all routines for our guestbook */
  class Guestbook extends SpecialPage {
    var $gbName, $gbOrigin, $gbEmail, $gbHomepage, $gbEntry, $gbCaptcha;

    function Guestbook() {
      SpecialPage::SpecialPage( 'Guestbook' );
      $this->includable(true);
      $this->Listed(false);
    }

    function execute() {
      global $wgOut, $wgRequest;

      $wgOut->setPagetitle($this->text("headline"));

      if ($wgRequest->getText( 'action' ) == "add") {
        if (!$this->ParseInput()) {
          $this->getForm(); return;
	}
      }

      $p=$wgRequest->getText( 'p' );
      if (is_numeric($p)) {
        $this->getEntries($p);
      } else {
        $this->getEntries(0);
      }

      return;
    }

    function success($msg) {
      global $wgOut;

      $wgOut->addHTML('<div class="successbox">');
      $wgOut->addHTML($msg);
      $wgOut->addHTML('</div><div class="visualClear"></div>');

      return;
    }

    function text($str) {
      global $wgGuestbookLanguage, $wgUser;
      require("lang.php");

      /* we may add some other languages here as well */
      switch($wgUser->getOption('language')) {
        case "de": $msg=&$wgGbMsgDe; break;
        default:   $msg=&$wgGbMsgEn; break;
      }

      if (!isset($msg[$str])) { return "error @ text($str)"; }

      return $msg[$str];
    }

    function ParseInput() {
      require("captcha-class.php");
      global $wgRequest;

      /* we are called the first time */
      $this->gbSubmit = $wgRequest->getText( 'gbSubmit' );
      if ($this->gbSubmit == "") return;

      /* required */
      $this->gbName = $wgRequest->getText( 'gbName' );
      $this->gbEntry = $wgRequest->getText( 'gbEntry' );
      $this->gbCaptcha = $wgRequest->getText( 'gbCaptcha' );

      /* optional */
      $this->gbOrigin = $wgRequest->getText( 'gbOrigin' );
      $this->gbEmail = $wgRequest->getText( 'gbEmail' );
      $this->gbHomepage = $wgRequest->getText( 'gbHomepage' );

      if ($this->gbName == "") {
        $this->error($this->text("err_name"));
	return;
      }
      if ($this->gbEntry == "") {
        $this->error($this->text("err_entry"));
	return;
      }
      if ($this->gbCaptcha == "") {
        $this->error($this->text("err_captcha"));
	return;
      }

      $cap = new captcha();
      if ($cap->verify($this->gbCaptcha) != true) {
        return $this->text("captcha_fail");
      }
      unset($cap);

      $gbTimestamp = time(0);
      $dbw =& wfGetDB( DB_MASTER );
      $dbw->insert('guestbook', array(
        'gb_timestamp'  => $gbTimestamp,
	'gb_name' => $this->gbName,
	'gb_entry' => $this->gbEntry,
	'gb_email' => $this->gbEmail,
	'gb_origin' => $this->gbOrigin,
	'gb_homepage' => $this->gbHomepage,
	'gb_ip' => $_SERVER['SERVER_ADDR'], 
	'gb_browser' => $_SERVER['HTTP_USER_AGENT']),
	'Guestbook::ParseInput', array('IGNORE'));

      $this->success($this->text("entry_okay"));
      return 1;
    }

    function error($msg) {
      global $wgOut;

      $wgOut->addHTML('<div class="errorbox">');
      $wgOut->addHTML($msg);
      $wgOut->addHTML('</div><div class="visualClear"></div>');

      return;
    }

    function getEntries($page) {
      global $wgOut, $wgUser, $wgScript;

      $dbr =& wfGetDB( DB_SLAVE );
      $gb = $dbr->tableName('guestbook');

      $sql="SELECT COUNT(*) AS total FROM $gb";
      $res = $dbr->query($sql, 'wfGuestbookSpecial');
      $obj = $dbr->fetchObject( $res );
      $dbr->freeResult( $res );
      $total = $obj->total;

      $limit=$wgUser->getOption('rows'); /* entries per page */
      $start=$limit*$page;
      if ($start >= $total) $start=$total-$limit;
      if ($start < 0) $start=0;

      $wgOut->addHTML( '<div class="gbook">' );
      $wgOut->addWikiText($this->text("new_entry"));
      $url="$wgScript" . "Special:Guestbook";
      for ($p=0; $limit*$p<$total; $p++);
      if ($p > 1) {
        $wgOut->addHTML("<b>");
        $wgOut->addWikiText("----");
        for ($p=0; $limit*$p<$total; $p++) {
          if ($p == $page) {
            $wgOut->addHTML("$p ");
	  } else {
            $wgOut->addHTML("<a href=\"$url&amp;p=$p\">$p</a> ");
	  }
        }
        $wgOut->addHTML("</b>");
      }
      $wgOut->addHTML( '</div>' );

      $sql="SELECT * FROM $gb ORDER BY gb_timestamp DESC LIMIT $start,$limit";
      $res = $dbr->query($sql, 'wfGuestbookSpecial');
      while ($obj = $dbr->fetchObject( $res )) {
	$name=$obj->gb_name;
        $ts=date("Y-m-d @ H:m:s", $obj->gb_timestamp);
	$origin=$obj->gb_origin;
	$entry=str_replace("\n", "<br />", $obj->gb_entry);
	$ip=$obj->gb_ip;
	$browser=$obj->gb_browser;
	
	/* create to homepage link */
	$hp = $obj->gb_homepage;
	if ($hp != "") {
	  if (strstr($hp,'://')) {
	    $url = $hp;
	  } else {
	    $url = 'http://' . $hp;
	  }
	  $hp_link="(<a href=\"$url\">$hp</a>)";
	} else {
	  $hp_link="";
	}
	
	/* create email link */
	$email=$obj->gb_email;
	if ($email != "") {
	  $email=str_replace("@", "_at_", $email);
	  $email=str_replace(".", "_dot_", $email);
	  $email=str_replace("-", "_dash_", $email);
	  $email_link="<a href=\"mailto:$email\"></a>";
	} else {
	  $email_link="";
	}

        $out .= <<<ENDFORM
<div class="gbook">
<table width="100%" cellspacing="0" cellpadding="0">
<tr>
 <td width="100%"><b>$name</b> $email_link $hp_link</td>
 <td nowrap><small>$ts</small></td>
</tr><tr>
 <td colspan="2">$entry</td>
</tr><tr>
 <td valign="bottom" colspan="2"><small>[$ip / $browser]</small></td>
</tr>
</table>
</div>
ENDFORM;
      }
      $dbr->freeResult( $res );
      $wgOut->addHTML($out);

      return;
    }

    function getForm() {
      global $wgScriptPath, $wgScript, $wgOut, $wgUser;

      $gb1=$this->text("f_name");
      $gb2=$this->text("f_origin");
      $gb3=$this->text("f_email");
      $gb4=$this->text("f_homepage");
      $gb5=$this->text("f_entry");
      $gb6=$this->text("f_captcha");
      $gb7=$this->text("show_entries");
      $lnk="$wgScript" . "Special:Guestbook";
      $cap="$wgScriptPath/extensions/gbook/captcha.php";
      $bt1=$this->text("b_submit");
      $bt2=$this->text("b_reset");
      $hl=$this->text("headline2");
      $lng=$wgUser->getOption('language');

      $out=<<<ENDFORM
<div id="gbook">
<h2>$hl</h2>
<form name="guestbook" method="post" action="">
<input type="hidden" name="gbSubmit" value="okay" />
<table width="100%" border="0">
<tr>
 <td valign="top">$gb1:</td>
 <td><input tabindex="1" type="text" name="gbName" size="40" value="$this->gbName" /></td>
 <td valign="top" rowspan="4" align="right">
 <p><a href="http://$lng.wikipedia.org/wiki/Captcha" title="what is captcha">$gb6</a>: <input tabindex="6" type="text" name="gbCaptcha" size="8" value="" />
 <p><img alt="captcha" src="$cap"></img>
 </td>
</tr><tr>
 <td valign="top">$gb2:</td>
 <td><input tabindex="2" type="text" name="gbOrigin" size="40" value="$this->gbOrigin" /></td>
</tr><tr>
 <td valign="top">$gb3:</td>
 <td><input tabindex="3" type="text" name="gbEmail" size="40" value="$this->gbEmail" /></td>
</tr><tr>
 <td valign="top">$gb4:</td>
 <td><input tabindex="4" type="text" name="gbHomepage" size="40" value="$this->gbHomepage" /></td>
</tr><tr>
 <td valign="top">$gb5:</td>
 <td colspan="2"><textarea tabindex="5" name="gbEntry" cols="40" rows="7">$this->gbEntry</textarea></td>
</tr><tr>
 <td colspan="2"><a href="$lnk">$gb7</a></td>
 <td align="right"><input tabindex="7" type="submit" value="$bt1"/><input tabindex="8" type="reset" value="$bt2"/></td>
</tr>
</table>
</form>
</div>
ENDFORM;
      $wgOut->addHTML( $out );

      return;
    }

  } /* class */

  SpecialPage::addPage( new Guestbook );
}

?>
